@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2018 Linutronix GmbH. All rights reserved.
+ * Copyright (c) 2012-2023 Linutronix GmbH. All rights reserved.
  *
  * SPDX-License-Identifier: BSD-2-Clause
  */
@@ -843,7 +843,7 @@ static int do_elf_ph_parse(struct dump_info *di, GElf_Phdr *type,
 
 		/* if flags specified, they must match */
 		if (type->p_flags) {
-			if ((phdr->p_flags & type->p_flags) != type->p_flags)
+			if ((phdr->p_flags & type->p_flags) == 0)
 				continue;
 		}
 
@@ -927,7 +927,7 @@ static int parse_vma_info(struct dump_info *di, size_t *phnum_found)
 	/* looking for readable loadable program segments */
 	memset(&type, 0, sizeof(type));
 	type.p_type = PT_LOAD;
-	type.p_flags = PF_R;
+	type.p_flags = PF_R | PF_W;
 	if (do_elf_ph_parse(di, &type, vma_cb, phnum_found) != 0)
 		return -1;
 
@@ -1996,6 +1996,14 @@ static int get_stack_pointer(pid_t pid, unsigned long *addr)
 		p = strchr(p, ' ');
 		if (!p)
 			goto out_err;
+		/*
+		 * Item 2 (comm) can contain spaces, which complicates the
+		 * parsing since a space is also the field delimeter. This
+		 * item is enclosed in parentheses, so do not continue until
+		 * a trailing parenthesis is seen.
+		 */
+		if (i == 1 && (*(p-1)) != ')')
+			i--;
 		p++;
 	}
 
@@ -2757,7 +2765,7 @@ static int dump_data_content(struct dump_info *di, struct mcd_dump_data *dd,
 	if (dd->ident) {
 		/* dump to external file */
 		ret = dump_data_content_file(di, dd);
-	} else {
+	} else if (di->core_fd >= 0) {
 		/* dump to core */
 		ret = dump_data_content_core(di, dd, symname);
 	}
@@ -3028,22 +3036,25 @@ static int get_robust_mutex_list(struct dump_info *di)
 	dump_vma(di, l_head, sizeof(struct robust_list_head), 0,
 		 "robust mutex head");
 
-	if (read_remote(di, l_head + offsetof(struct robust_list_head, list),
-			&l_start, sizeof(l_start)) != 0) {
-		return 1;
-	}
-
+	l_start = l_head + offsetof(struct robust_list_head, list);
 	l_tmp = l_start;
-	do {
-		dump_vma(di, l_tmp, sizeof(struct robust_list), 0,
-			 "robust mutex");
 
+	while (1) {
 		if (read_remote(di, l_tmp + offsetof(struct robust_list, next),
 				&l_tmp, sizeof(l_tmp)) != 0) {
 			return 1;
 		}
 
-	} while (l_tmp != l_start);
+		/* glibc uses bit0 of pointer to flag PI mutex */
+		l_tmp = l_tmp & ~1UL;
+
+		/* the last list item points back to the start */
+		if (l_tmp == l_start)
+			break;
+
+		dump_vma(di, l_tmp, sizeof(struct robust_list), 0,
+			 "robust mutex");
+	}
 
 	return 0;
 }
